/* tslint:disable:max-line-length */
import {fakeAsync, TestBed, tick} from '@angular/core/testing';
import {AppComponent} from './app.component';
import {DataSourceService} from './services/data-source.service';
import {MatDialog} from '@angular/material/dialog';
import {of} from 'rxjs';
import {HttpClientTestingModule} from '@angular/common/http/testing';
import {UserContextService} from './services/user-context.service';
import {NamePipe} from './patient-information/name.pipe';
import {CprPipe} from './pipes/cpr.pipe';
import {AppModule} from './app.module';
import {PatientContextService} from './services/patient-context.service';
import {MatIconTestingModule} from '@angular/material/icon/testing';
import {DataCardResponse} from 'facade-api';

describe('AppComponent', () => {
  let patientContext: PatientContextService;
  let dataCard: DataCardResponse;

  beforeEach(async () => {
    dataCard = {
      identifier: '1',
      episodeOfCareIdentifier: '2',
      documentStatus: 'ACTIVE',
      recordTarget: {
        identifier: '1008821234',
        name: {
          firstNames: ['Peter'],
          middleNames: [],
          lastName: 'Parker'
        }
      },
      personalDataSection: {
        cpr: '1008821234',
        name: {
          firstNames: ['Peter'],
          middleNames: [],
          lastName: 'Parker'
        }
      },
      socialHistorySection: {
        partner: {
          translationLanguage: 'something',
          nationality: 'whetever',
          needsTranslation: 'yes'
        }
      },
      expectedDateOfDeliverySection: {
        expectedDateOfDeliveryFromLastPeriod: {}
      }
    } as any;
    const dataSourceService = {
      allDataCards$: of([dataCard]),
      isLoading$: of(false)
    };
    const userContextService = {
      selectedRole$: of({authorizationCode: 'LK34Q', educationCode: '9495', educationType: 'Bandagist'}),
      userContext$: of({commonName: 'Lars Larsen', organizationName: 'Company A/S', eligibleRoles: [{authorizationCode: 'J0184', educationCode: '7170', educationType: 'Læge'}, {authorizationCode: 'LK34Q', educationCode: '9495', educationType: 'Bandagist'}]}),
      eligibleRoles$: of({authorizationCode: 'LK34Q', educationCode: '9495', educationType: 'Bandagist'}),
      contextIsInitialized$: of(true),
      selectedOrganizationSor$: of('sorId')
    };
    patientContext = new PatientContextService({nativeWindow: () => ({
        document: {cookie: '2394'},
        localStorage: {
          getItem: () => '1234567890',
          setItem: () => undefined,
        }
      })});

    await TestBed.configureTestingModule({
      declarations: [ AppComponent, NamePipe, CprPipe ],
      imports: [
        HttpClientTestingModule,
        MatIconTestingModule,
        AppModule,
      ],
      providers: [
        {provide: DataSourceService, useValue: dataSourceService},
        {provide: MatDialog, useValue: {}},
        {provide: UserContextService, useValue: userContextService},
        {provide: PatientContextService, useValue: patientContext}
      ]
    })
      .compileComponents();
  });

  it('should create the app', () => {
    const fixture = TestBed.createComponent(AppComponent);
    const app = fixture.componentInstance;
    expect(app).toBeTruthy();
  });

  it(`shows correct title`, () => {
    const fixture = TestBed.createComponent(AppComponent);
    const app = fixture.componentInstance;
    expect(app.title).toEqual('Graviditetsportalen');
  });

  it('should render title', fakeAsync(() => {
    const fixture = TestBed.createComponent(AppComponent);
    fixture.detectChanges();
    const compiled = fixture.nativeElement;
    tick(200);
    expect(compiled.querySelector('.patientContext > .content > .patientName').textContent).toContain('Peter Parker');
  }));

  it('should render column tolkebistand', fakeAsync(() => {
    const fixture = TestBed.createComponent(AppComponent);
    fixture.detectChanges();
    const compiled = fixture.nativeElement;
    tick();
    expect(compiled.querySelector('.patientContext > .content > .patientInterpretingAssistance').textContent).toBeTruthy();
  }));

  it('sets active episode of care if first datacard is active', () => {
    const spy = spyOn(patientContext, 'setEpisodeOfCare');
    const fixture = TestBed.createComponent(AppComponent);
    const component = fixture.componentInstance;
    component.ngOnInit();
    expect(spy).toHaveBeenCalledWith('2');
  });

  it('does not set active episode of care if first datacard is NEW', () => {
    dataCard.documentStatus = 'NEW';
    const spy = spyOn(patientContext, 'setEpisodeOfCare');
    const fixture = TestBed.createComponent(AppComponent);
    const component = fixture.componentInstance;
    component.ngOnInit();
    expect(spy).not.toHaveBeenCalled();
  });

  it('does not set active episode of care if first datacard is CLOSED', () => {
    dataCard.documentStatus = 'CLOSED';
    const spy = spyOn(patientContext, 'setEpisodeOfCare');
    const fixture = TestBed.createComponent(AppComponent);
    const component = fixture.componentInstance;
    component.ngOnInit();
    expect(spy).not.toHaveBeenCalled();
  });

  describe('when patientContext.hasPrivatelyMarkedData is false', () => {
    beforeEach(() => {
      patientContext.setHasPrivatelyMarkedData(false);
      patientContext.setXBreakTheGlassReason(undefined);
    });

    it('should render column tolkebistand', fakeAsync(() => {
      const fixture = TestBed.createComponent(AppComponent);
      fixture.detectChanges();
      const compiled = fixture.nativeElement;
      tick();
      expect(compiled.querySelector('.patientContext > .content > .patientInterpretingAssistance')).toBeTruthy();
    }));
  });

  describe('when patientContext.hasPrivatelyMarkedData is undefined', () => {
    beforeEach(() => {
      patientContext.setHasPrivatelyMarkedData(undefined);
      patientContext.setXBreakTheGlassReason(undefined);
    });

    it('should render column tolkebistand', fakeAsync(() => {
      const fixture = TestBed.createComponent(AppComponent);
      fixture.detectChanges();
      const compiled = fixture.nativeElement;
      tick();
      expect(compiled.querySelector('.patientContext > .content > .patientInterpretingAssistance')).toBeTruthy();
    }));
  });

  describe('when patientContext.hasPrivatelyMarkedData is true', () => {
    beforeEach(() => {
      patientContext.setHasPrivatelyMarkedData(true);
      patientContext.setXBreakTheGlassReason(undefined);
    });

    it('should not render column tolkebistand', fakeAsync(() => {
      const fixture = TestBed.createComponent(AppComponent);
      fixture.detectChanges();
      const compiled = fixture.nativeElement;
      tick();
      expect(compiled.querySelector('.patientContext > .content > .patientInterpretingAssistance')).toBeFalsy();
    }));

    describe('and patientContext.xBreakTheGlassReason is truthy', () => {
      beforeEach(() => {
        patientContext.setXBreakTheGlassReason('stangspring');
      });

      it('should render column tolkebistand', fakeAsync(() => {
        const fixture = TestBed.createComponent(AppComponent);
        fixture.detectChanges();
        const compiled = fixture.nativeElement;
        tick();
        expect(compiled.querySelector('.patientContext > .content > .patientInterpretingAssistance')).toBeTruthy();
      }));
    });
  });
});
